# This file was auto-generated by Fern from our API Definition.

from .create_voice_preset_request import CreateVoicePresetRequest
from .create_voice_preset_response import CreateVoicePresetResponse
from .delete_voice_preset_response import DeleteVoicePresetResponse
from .generate_voice_sample_request import GenerateVoiceSampleRequest
from .generate_voice_sample_response import GenerateVoiceSampleResponse
from .list_voice_options_response import ListVoiceOptionsResponse
from .optimize_for import OptimizeFor
from .preset import Preset
from .publish_voice_preset_response import PublishVoicePresetResponse
from .retrieve_voice_preset_response import RetrieveVoicePresetResponse
from .retrieved_voice import RetrievedVoice
from .update_voice_preset_request import UpdateVoicePresetRequest
from .update_voice_preset_response import UpdateVoicePresetResponse
from .voice import Voice
from .voice_settings_data import VoiceSettingsData
from .voice_updates import VoiceUpdates

__all__ = [
    "CreateVoicePresetRequest",
    "CreateVoicePresetResponse",
    "DeleteVoicePresetResponse",
    "GenerateVoiceSampleRequest",
    "GenerateVoiceSampleResponse",
    "ListVoiceOptionsResponse",
    "OptimizeFor",
    "Preset",
    "PublishVoicePresetResponse",
    "RetrieveVoicePresetResponse",
    "RetrievedVoice",
    "UpdateVoicePresetRequest",
    "UpdateVoicePresetResponse",
    "Voice",
    "VoiceSettingsData",
    "VoiceUpdates",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime
from .goal import Goal

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class AnalyzeCall(pydantic.BaseModel):
    goal: Goal = pydantic.Field()
    """
    This is the overall purpose of the call. Provides context for the analysis to guide how the questions/transcripts are interpreted.
    """

    questions: typing.List[typing.List[str]] = pydantic.Field()
    """
    An array of questions to be analyzed for the call.
    
    Each question should be an array with two elements - the question text and the expected answer type (e.g., “string”, “boolean”).
    
    Fairly flexible in terms of the expected answer type, and unanswerable questions will default to `null`.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .types import (
    AnalyzeCall,
    AnalyzeCallResponse,
    AnsweredByEnum,
    AudioRecordingResponse,
    CallDetailsResponse,
    CallIdParam,
    Goal,
    ListCallsResponse,
    ListedCallData,
    StopCallResponse,
    Transcript,
    UserEnum,
)

__all__ = [
    "AnalyzeCall",
    "AnalyzeCallResponse",
    "AnsweredByEnum",
    "AudioRecordingResponse",
    "CallDetailsResponse",
    "CallIdParam",
    "Goal",
    "ListCallsResponse",
    "ListedCallData",
    "StopCallResponse",
    "Transcript",
    "UserEnum",
]



# This file was auto-generated by Fern from our API Definition.

from .dynamic_data import DynamicData
from .interruption_threshold import InterruptionThreshold
from .method_enum import MethodEnum
from .model_enum import ModelEnum
from .pronunciation_object import PronunciationObject
from .prounciation_guide import ProunciationGuide
from .response_data import ResponseData
from .send_call import SendCall
from .send_call_response import SendCallResponse
from .temperature import Temperature
from .transfer_list import TransferList
from .voicemail_action import VoicemailAction

__all__ = [
    "DynamicData",
    "InterruptionThreshold",
    "MethodEnum",
    "ModelEnum",
    "PronunciationObject",
    "ProunciationGuide",
    "ResponseData",
    "SendCall",
    "SendCallResponse",
    "Temperature",
    "TransferList",
    "VoicemailAction",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...commons.types.status import Status
from ...core.datetime_utils import serialize_datetime

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class AnalyzeCallResponse(pydantic.BaseModel):
    status: Status = pydantic.Field()
    """
    Will be `success` if the request was successful.
    """

    message: str = pydantic.Field()
    """
    Confirms the request was successful, or provides an error message if the request failed.
    """

    answers: typing.List[typing.List[typing.Any]] = pydantic.Field()
    """
    Contains the analyzed answers for the call in an array.
    """

    credits_used: int = pydantic.Field()
    """
    Token-based price for the analysis request.
    
    As a rough estimate, the base cost is `0.003` credits with an additional `0.0015` credits per call in the call.
    
    Longer call transcripts and higher numbers of questions can increase the cost, however the cost scales very effectively with calls vs. individual calls.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .types import (
    A2PStatusEnum,
    CheckStatusRequest,
    CheckStatusResponse,
    GetMessagesRequest,
    GetMessagesResponse,
    LegalStructureEnum,
    RegisterRequest,
    RegisterResponse,
    ToggleReplyEnum,
    ToggleReplyRequest,
    ToggleReplyResponse,
    TrustedUser,
    UpdatePromptRequest,
    UpdatePromptResponse,
    UpdateWebhookRequest,
    UpdateWebhookResponse,
    VerticalEnum,
)

__all__ = [
    "A2PStatusEnum",
    "CheckStatusRequest",
    "CheckStatusResponse",
    "GetMessagesRequest",
    "GetMessagesResponse",
    "LegalStructureEnum",
    "RegisterRequest",
    "RegisterResponse",
    "ToggleReplyEnum",
    "ToggleReplyRequest",
    "ToggleReplyResponse",
    "TrustedUser",
    "UpdatePromptRequest",
    "UpdatePromptResponse",
    "UpdateWebhookRequest",
    "UpdateWebhookResponse",
    "VerticalEnum",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime
from .answered_by_enum import AnsweredByEnum
from .transcript import Transcript

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class CallDetailsResponse(pydantic.BaseModel):
    transcripts: typing.List[Transcript] = pydantic.Field()
    """
    An array of phrases spoken during the call.
    """

    variables: typing.Dict[str, str] = pydantic.Field()
    """
    Variables created during the call - both system variables as well as generated with `dynamic_data`.
    
    For example, if you used a `dynamic_data` API request to generate a variable called `appointment_time`, you would see it here.
    """

    concatenated_transcript: str = pydantic.Field()
    """
    A single string containing all of the text from the call. Excludes system messages and auto-generated data.
    """

    to: str = pydantic.Field()
    """
    The phone number that received the call.
    """

    from_: str = pydantic.Field(alias="from")
    """
    The phone number that made the call.
    """

    batch_id: str = pydantic.Field()
    """
    If the call is part of a batch, it's batch_id will be here.
    """

    request_data: typing.Dict[str, str] = pydantic.Field()
    """
    Details about parameters in the original api request.
    """

    completed: bool = pydantic.Field()
    """
    Whether the call has been completed. If it differs from the value of `queue_status`, this will be the most up-to-date status.
    """

    queue_status: str = pydantic.Field()
    """
    The status of the call. During extremely high volume periods, calls may be queued for a short period of time before being dispatched.
    
    Progresses through the following stages -
    
    - `new` - An API request has been received.
    - `queued` - Call pararameters have been validated and authentication succeeded.
    - `allocated` - Extremely brief, the call is being dispatched.
    - `started` - The phone call is live and in progress.
    - `complete` - The phone call has ended successfully.
    
    The following statuses show the point that was reached before an error -
    
    - `pre_queue_error` - An error occurred before the call was queued. Invalid parameters generally cause this.
    - `queue_error` - Error occurred while the call was queued. Ex. Valid phone number but to an unserviced area.
    - `call_error` - Error occurred during live call. May be caused by transferring to an invalid phone number or an unforeseen error.
    - `complete_error` - Error occurred after the call was completed. Ex. A post-call webhook failed.
    
    If at any point an error occurs, it will be recorded in the `error_message` field.
    """

    error_message: str = pydantic.Field()
    """
    If an error occurs, this will contain a description of the error. Otherwise, it will be null.
    """

    answered_by: AnsweredByEnum = pydantic.Field()
    """
    If `answered_by_enabled` was set to `true` in the original API request, this field contains one of the following values:
    
    - `human`: The call was answered by a human.
    - `voicemail`: The call was answered by an answering machine or voicemail.
    - `unknown`: There was not enough audio at the start of the call to make a determination.
    - `no-answer`: The call was not answered.
    - `null`: Not enabled, or still processing the result.
    """

    endpoint_url: str = pydantic.Field()
    """
    The URL that was called to dispatch the phone call.
    """

    max_duration: float = pydantic.Field()
    """
    The maximum length of time the call was allowed to last. If the call would exceed this length, it's ended early.
    """

    corrected_duration: float = pydantic.Field()
    """
    The total length of time the call was connected. This differs from `call_length` in that it includes ringing and connection time.
    """

    call_length: float = pydantic.Field()
    """
    The length of the call in minutes.
    """

    created_at: dt.datetime = pydantic.Field()
    """
    The timestamp for when the call was dispatched.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...commons.types.phone_number import PhoneNumber
from ...commons.types.tools import Tools
from ...commons.types.voice_id import VoiceId
from ...commons.types.webhook import Webhook
from ...core.datetime_utils import serialize_datetime
from .model import Model
from .voice_settings import VoiceSettings

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class UpdateInbound(pydantic.BaseModel):
    """
    from blandai import UpdateInbound

    UpdateInbound(
        prompt="29382721828",
        transfer_list={
            "default": "+1234567890",
            "sales": "+1234567890",
            "support": "+1234567890",
            "billing": "+1234567890",
        },
        model="enhanced",
    )
    """

    prompt: str = pydantic.Field()
    """
    Provide instructions, relevant information, and examples of the ideal conversation flow.
    
    For inbound numbers, consider including additional context about the purpose of the call, and what types of callers to expect.
    
    **Best Practices:**
    
    **Out-of-the-Box Behaviors (Summarized):**
    
    - Speech pattern: Direct, concise, casual
    - Spells out symbols, acronyms, abbreviations, percentages, etc. ($4,000,000 -> “four million dollars”)
    - Asks clarifying questions
    
    **Prompting Tips:**
    
    Want to easily test out exactly how your agent will behave?
    
    - Try out Agent Testing!
    - Aim for less than 2,000 characters where possible.
    - Simple, direct prompts are the most predictable and reliable.
    - Frame instructions positively:
      - `"Do this"` rather than `"Don't do this"`.
      - Ex. `“Keep the conversation casual”` rather than `“Don't be too formal”`.
      - This gives concrete examples of what to do, instead of leaving expected behavior open to interpretation.
    """

    transfer_list: typing.Optional[typing.Dict[str, str]] = pydantic.Field(default=None)
    """
    Give your agent the ability to transfer calls to a set of phone numbers.
    
    Overrides `transfer_phone_number` if a `transfer_list.default` is specified.
    
    Will default to `transfer_list.default`, or the chosen phone number.
    """

    model: typing.Optional[Model] = pydantic.Field(default=None)
    """
    Select a model to use for your call.
    
    If no model is provided, the default value `enhanced` will be used.
    
    In nearly all cases, `enhanced` is the best choice for now.
    
    Model Differences:
    
    There are three different ways to use Bland:
    
    - `model: base`
    
      - The original, follows scripts/procedures most effectively.
      - Supports all features and capabilities.
      - Best for Custom Tools
    
    - `model: enhanced`
    
      - Much faster latency and very conversational, works best with objective-based prompts.
      - Supports all features and capabilities.
    
    - `model: turbo`
      - The absolute fastest latency possible, can be verbose at times
      - Limited capabilities currently (excludes Transferring, IVR navigation, Custom Tools)
      - Extremely realistic conversation capabilities
    """

    transfer_phone_number: typing.Optional[PhoneNumber] = pydantic.Field(default=None)
    """
    A phone number that the agent can transfer to under specific conditions - such as being asked to speak to a human or supervisor.
    
    Set to `null` to remove.
    
    Prompting Notes:
    
    - Specify conditions that the agent should transfer to a human under (examples are great!)
    - In the `task`, refer to the action solely as “transfer” or “transferring”.
    - Alternate phrasing such as “swap” or “switch” can mislead the agent, causing the action to be ignored.
    """

    voice_id: typing.Optional[VoiceId] = None
    webhook: typing.Optional[Webhook] = None
    record: typing.Optional[bool] = pydantic.Field(default=None)
    """
    To record your phone call, set record to `true`. When your call completes, you can access through the `recording_url` field in the call details or your webhook.
    
    If no value is provided, the default value `false` will be used.
    """

    first_sentence: typing.Optional[str] = pydantic.Field(default=None)
    """
    A phrase that your call will start with instead of a generating one on the fly. This works both with and without `wait_for_greeting`. Can be more than one sentence, but must be less than 200 characters.
    
    To remove, set to `null` or an empty string.
    """

    tools: Tools
    voice_settings: typing.Optional[VoiceSettings] = pydantic.Field(default=None)
    """
    Alter advanced voice settings for your agent.
    
    To remove, set to `null` or an empty string.
    """

    dynamic_data: typing.Optional[str] = pydantic.Field(default=None)
    """
    Integrate data from external APIs into your agent's knowledge.
    
    Set to `null` or an empty string to clear dynamic data settings.
    
    Detailed usage in the Send Call endpoint.
    """

    interruption_threshold: typing.Optional[int] = pydantic.Field(default=None)
    """
    How long our AI phone agent should wait in milliseconds before responding.
    
    When you increase the interruption latency, you force the AI phone agent to listen longer before responding. In practice, increasing the threshold results in less interruptions and more latency.
    
    Try setting the threshold to 500 milliseconds. You'll encounter higher latency, but you'll be interrupted less frequently.
    
    Set to `null` to reset to default.
    
    If no value is provided, the default value `50` will be used.
    """

    max_duration: typing.Optional[int] = pydantic.Field(default=None)
    """
    The maximum duration in minutes that calls to your agent can last before being automatically terminated.
    
    Set to `null` to reset to default.
    
    If no value is provided, the default value `30` will be used.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime
from .legal_structure_enum import LegalStructureEnum
from .trusted_user import TrustedUser
from .vertical_enum import VerticalEnum

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class RegisterRequest(pydantic.BaseModel):
    business_name: str = pydantic.Field(alias="businessName")
    """
    The legal name of your business.
    """

    ein: str = pydantic.Field()
    """
    Your Employer Identification Number.
    """

    vertical: VerticalEnum = pydantic.Field()
    """
    The industry vertical of your business.
    """

    address: str = pydantic.Field()
    """
    The business address.
    """

    city: str = pydantic.Field()
    """
    The city of your business.
    """

    state: str = pydantic.Field()
    """
    The state of your business. Must be a valid US state code.
    """

    postal_code: str = pydantic.Field(alias="postalCode")
    """
    The postal code of your business.
    """

    country: str = pydantic.Field()
    """
    The country of your business.
    """

    email: str = pydantic.Field()
    """
    The email address for your business.
    """

    type: LegalStructureEnum = pydantic.Field()
    """
    Legal structure of the business.
    """

    website: str = pydantic.Field()
    """
    Your business's website URL.
    """

    opt_in_info: str = pydantic.Field()
    """
    Information regarding opt-in procedures for your messaging service.
    
    EX: “Customers must explicitly consent on our website and during the phone call.”
    """

    message_samples: typing.List[str] = pydantic.Field(alias="messageSamples")
    """
    An array of three strings, each a sample message you plan to use.
    """

    trusted_user: TrustedUser = pydantic.Field()
    """
    An object containing details about the trusted user registering the brand.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime
from .call_length_summary import CallLengthSummary

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class CallLength(pydantic.BaseModel):
    average: int = pydantic.Field()
    """
    The average call length in minutes.
    """

    average_nonzero: int = pydantic.Field()
    """
    The average call length in minutes, excluding calls with a length of less than one second.
    """

    summary: CallLengthSummary = pydantic.Field()
    """
    A summary of the call lengths, grouped into ranges.
    """

    all: typing.List[int] = pydantic.Field()
    """
    Contains each call length, in case you want to use a different grouping than the default.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

from ..calls.types.analyze_call_response import AnalyzeCallResponse
from ..commons.errors.server_error import ServerError
from ..commons.errors.unauthorized_error import UnauthorizedError
from ..commons.types.error_body import ErrorBody
from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.jsonable_encoder import jsonable_encoder
from ..core.remove_none_from_dict import remove_none_from_dict
from ..core.request_options import RequestOptions
from .types.analyze_batch_request import AnalyzeBatchRequest
from .types.batch_details_response import BatchDetailsResponse
from .types.list_batches_response import ListBatchesResponse
from .types.retrieve_batch_analysis_response import RetrieveBatchAnalysisResponse
from .types.send_batch_request import SendBatchRequest
from .types.send_batch_response import SendBatchResponse
from .types.stop_active_batch_response import StopActiveBatchResponse

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class BatchesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def send(
        self, *, request: SendBatchRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> SendBatchResponse:
        """
        Send large volumes of calls at once with a single API request.

        Parameters:
            - request: SendBatchRequest.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/batches"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(SendBatchResponse, _response.json())  # type: ignore
        if _response.status_code == 500:
            raise ServerError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def analyze(
        self, batch_id: str, *, request: AnalyzeBatchRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> AnalyzeCallResponse:
        """
        Analyzes a batch of calls based on using questions and goals.

        Parameters:
            - batch_id: str. The unique identifier for the batch of calls to be analyzed.

            - request: AnalyzeBatchRequest.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"v1/batches/{jsonable_encoder(batch_id)}/analyze"
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(AnalyzeCallResponse, _response.json())  # type: ignore
        if _response.status_code == 500:
            raise ServerError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def stop(
        self, batch_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> StopActiveBatchResponse:
        """
        Stops all active calls in a batch.

        Parameters:
            - batch_id: str. The unique identifier for the batch of calls to be cancelled.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"v1/batches/{jsonable_encoder(batch_id)}/stop"
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))
            if request_options is not None
            else None,
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(StopActiveBatchResponse, _response.json())  # type: ignore
        if _response.status_code == 500:
            raise ServerError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def list(self, *, request_options: typing.Optional[RequestOptions] = None) -> ListBatchesResponse:
        """
        Retrieves batch-specific data for each batch you've created.

        Parameters:
            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/batches"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ListBatchesResponse, _response.json())  # type: ignore
        if _response.status_code == 500:
            raise ServerError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def details(
        self,
        batch_id: str,
        *,
        include_calls: typing.Optional[bool] = None,
        include_transcripts: typing.Optional[bool] = None,
        include_analysis: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BatchDetailsResponse:
        """
        Retrieves calls and batch data for a specific batch_id.

        Parameters:
            - batch_id: str. The unique identifier for the batch of calls you want to retrieve.

            - include_calls: typing.Optional[bool]. Whether or not to include individual call data in the response.

                                                    If no value is provided, `include_calls` defaults to `true`.
            - include_transcripts: typing.Optional[bool]. If calls are included, can be set to false to exclude transcripts from the response.

                                                          If no value is provided, `include_transcripts` defaults to `true`.
            - include_analysis: typing.Optional[bool]. If calls are included, can be set to false to exclude analysis from the response.

                                                       If no value is provided, `include_analysis` defaults to `true`.
            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"v1/batches/{jsonable_encoder(batch_id)}"),
            params=jsonable_encoder(
                remove_none_from_dict(
                    {
                        "include_calls": include_calls,
                        "include_transcripts": include_transcripts,
                        "include_analysis": include_analysis,
                        **(
                            request_options.get("additional_query_parameters", {})
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(BatchDetailsResponse, _response.json())  # type: ignore
        if _response.status_code == 500:
            raise ServerError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def retrieve(
        self, batch_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> RetrieveBatchAnalysisResponse:
        """
        Retrieves the analyses for a specific batch of calls, including details of each call's analysis.

        Parameters:
            - batch_id: str. The unique identifier for the call batch. Returned in the response when creating a batch, or when listing all batches.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"v1/batches/{jsonable_encoder(batch_id)}/analysis"
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(RetrieveBatchAnalysisResponse, _response.json())  # type: ignore
        if _response.status_code == 500:
            raise ServerError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncBatchesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def send(
        self, *, request: SendBatchRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> SendBatchResponse:
        """
        Send large volumes of calls at once with a single API request.

        Parameters:
            - request: SendBatchRequest.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/batches"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(SendBatchResponse, _response.json())  # type: ignore
        if _response.status_code == 500:
            raise ServerError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def analyze(
        self, batch_id: str, *, request: AnalyzeBatchRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> AnalyzeCallResponse:
        """
        Analyzes a batch of calls based on using questions and goals.

        Parameters:
            - batch_id: str. The unique identifier for the batch of calls to be analyzed.

            - request: AnalyzeBatchRequest.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"v1/batches/{jsonable_encoder(batch_id)}/analyze"
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(AnalyzeCallResponse, _response.json())  # type: ignore
        if _response.status_code == 500:
            raise ServerError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def stop(
        self, batch_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> StopActiveBatchResponse:
        """
        Stops all active calls in a batch.

        Parameters:
            - batch_id: str. The unique identifier for the batch of calls to be cancelled.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"v1/batches/{jsonable_encoder(batch_id)}/stop"
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))
            if request_options is not None
            else None,
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(StopActiveBatchResponse, _response.json())  # type: ignore
        if _response.status_code == 500:
            raise ServerError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def list(self, *, request_options: typing.Optional[RequestOptions] = None) -> ListBatchesResponse:
        """
        Retrieves batch-specific data for each batch you've created.

        Parameters:
            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/batches"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ListBatchesResponse, _response.json())  # type: ignore
        if _response.status_code == 500:
            raise ServerError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def details(
        self,
        batch_id: str,
        *,
        include_calls: typing.Optional[bool] = None,
        include_transcripts: typing.Optional[bool] = None,
        include_analysis: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BatchDetailsResponse:
        """
        Retrieves calls and batch data for a specific batch_id.

        Parameters:
            - batch_id: str. The unique identifier for the batch of calls you want to retrieve.

            - include_calls: typing.Optional[bool]. Whether or not to include individual call data in the response.

                                                    If no value is provided, `include_calls` defaults to `true`.
            - include_transcripts: typing.Optional[bool]. If calls are included, can be set to false to exclude transcripts from the response.

                                                          If no value is provided, `include_transcripts` defaults to `true`.
            - include_analysis: typing.Optional[bool]. If calls are included, can be set to false to exclude analysis from the response.

                                                       If no value is provided, `include_analysis` defaults to `true`.
            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"v1/batches/{jsonable_encoder(batch_id)}"),
            params=jsonable_encoder(
                remove_none_from_dict(
                    {
                        "include_calls": include_calls,
                        "include_transcripts": include_transcripts,
                        "include_analysis": include_analysis,
                        **(
                            request_options.get("additional_query_parameters", {})
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(BatchDetailsResponse, _response.json())  # type: ignore
        if _response.status_code == 500:
            raise ServerError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def retrieve(
        self, batch_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> RetrieveBatchAnalysisResponse:
        """
        Retrieves the analyses for a specific batch of calls, including details of each call's analysis.

        Parameters:
            - batch_id: str. The unique identifier for the call batch. Returned in the response when creating a batch, or when listing all batches.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"v1/batches/{jsonable_encoder(batch_id)}/analysis"
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(RetrieveBatchAnalysisResponse, _response.json())  # type: ignore
        if _response.status_code == 500:
            raise ServerError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from .analyze_call import AnalyzeCall
from .analyze_call_response import AnalyzeCallResponse
from .answered_by_enum import AnsweredByEnum
from .audio_recording_response import AudioRecordingResponse
from .call_details_response import CallDetailsResponse
from .call_id_param import CallIdParam
from .goal import Goal
from .list_calls_response import ListCallsResponse
from .listed_call_data import ListedCallData
from .stop_call_response import StopCallResponse
from .transcript import Transcript
from .user_enum import UserEnum

__all__ = [
    "AnalyzeCall",
    "AnalyzeCallResponse",
    "AnsweredByEnum",
    "AudioRecordingResponse",
    "CallDetailsResponse",
    "CallIdParam",
    "Goal",
    "ListCallsResponse",
    "ListedCallData",
    "StopCallResponse",
    "Transcript",
    "UserEnum",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime
from .call_length import CallLength
from .queue_status import QueueStatus

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class AnalysisObject(pydantic.BaseModel):
    """
    from blandai import AnalysisObject, CallLength, CallLengthSummary, QueueStatus

    AnalysisObject(
        total_calls=3,
        completed_calls=2,
        in_progress_calls=1,
        queue_statuses=QueueStatus(
            complete=2,
            queued=1,
            call_error=0,
        ),
        call_lengths=CallLength(
            average=7,
            average_nonzero=7,
            summary=CallLengthSummary(
                _0_5=2,
                _5_10=0,
                _10_15=1,
                _15_20=0,
                _20_30=0,
                _30_45=0,
                _45_60=0,
                _60_90=0,
                _90_120=0,
                _120=0,
            ),
            all=[5, 5, 11],
        ),
        call_ids=[
            "73c809a9-db35-4edb-8fa6-95a6a942daf8",
            "73c809a9-db35-4edb-8fa6-95a6a942daf9",
            "73c809a9-db35-4edb-8fa6-95a6a942daf0",
        ],
        error_messages=[
            "Cannot transfer to +12223334444 - Call is no longer active."
        ],
        endpoints={"API": "3"},
    )
    """

    total_calls: int = pydantic.Field()
    """
    The total number of calls in the batch, including completed and in-progress calls.
    """

    completed_calls: int = pydantic.Field()
    """
    The total number of completed calls in the batch.
    """

    in_progress_calls: int = pydantic.Field()
    """
    The total number of in-progress calls in the batch.
    """

    queue_statuses: QueueStatus = pydantic.Field()
    """
    An object containing the number of calls in each queue status.
    """

    call_lengths: CallLength = pydantic.Field()
    """
    Contains `average`, `average_nonzero`, `summary` and `all` fields.
    """

    call_ids: typing.List[str] = pydantic.Field()
    """
    Contains each `call_id` in the batch.
    """

    error_messages: typing.List[str] = pydantic.Field()
    """
    Contains any error messages that calls in the batch may have.
    """

    endpoints: typing.Dict[str, str] = pydantic.Field()
    """
    Contains the number of calls that have been sent to each endpoint. Applicable only to API integrations.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class OutboundPurchase(pydantic.BaseModel):
    area_code: typing.Optional[str] = pydantic.Field(default=None)
    """
    Choose a three-digit area code for your phone number. If set as a parameter, a number will only be purchased by exact match if available.
    
    If no area code is set, the default `415` area code will be used.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...commons.types.status import Status
from ...core.datetime_utils import serialize_datetime

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class AudioRecordingResponse(pydantic.BaseModel):
    status: Status
    message: typing.Optional[str] = pydantic.Field(default=None)
    """
    If the status is `success`, the `url` field will be present instead of this field.
    
    A 404 error will be returned if the call does not exist or the recording is not available. We can only retrieve recordings if the call was created with `record` set to `true`.
    
    A 400/500 error will be returned if there is an error retrieving the recording.
    """

    url: typing.Optional[str] = pydantic.Field(default=None)
    """
    If the status is `success`, the `url` will provide the exact location of the MP3 file storing the call's audio.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .list_outbound_response import ListOutboundResponse
from .outbound_number import OutboundNumber
from .outbound_purchase import OutboundPurchase
from .outbound_purchase_response import OutboundPurchaseResponse

__all__ = ["ListOutboundResponse", "OutboundNumber", "OutboundPurchase", "OutboundPurchaseResponse"]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime
from ...types.send_call import SendCall
from .batch_call_data import BatchCallData

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class SendBatchRequest(SendCall):
    """
    from blandai import BatchCallData, SendBatchRequest

    SendBatchRequest(
        phone_number="29382721828",
        task="Would love for you to check out our AI API!",
        base_prompt="You are calling a business to renew their subscription to a service before it expires on a date.",
        call_data=[
            BatchCallData(
                phone_number="1234567890",
                business="ABC Corp",
                service="Netflix",
                date="September 4th",
            ),
            BatchCallData(
                phone_number="32176540987",
                business="XYZ inc.",
                service="Window Cleaning",
                date="December 20th",
            ),
        ],
        label="Subscription Renewal",
        campaign_id="1234",
        test_mode=True,
    )
    """

    base_prompt: str = pydantic.Field()
    """
    This is the prompt or task used for all the phone calls in the request.
    
    Information can be inserted into it surrounding variable names with double curly braces.
    """

    call_data: typing.List[BatchCallData] = pydantic.Field()
    """
    Define a list of calls to make and their properties.
    
    Each call in call_data **MUST** have a `phone_number` property. Properties are case-sensitive.
    """

    label: typing.Optional[str] = pydantic.Field(default=None)
    """
    Adds a user-friendly label to your batch to keep track of it's original intention. This can help differentiate multiple call batches that are part of the same Campaign. Shown when a batch is retreived.
    """

    campaign_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Use `campaign_id` to organize related batches together. This can be set manually or auto-generated through Campaigns.
    """

    test_mode: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When this is set to `true`, only the first call of `call_data` will be dispatched. A common use case is to set the first `phone_number` value to your own to confirm everything's set up properly.
    
    Includes additional information in the response when true so that it's easier to find any issues.
    
    If no value is provided, `test_mode` defaults to `false`.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .analysis_object import AnalysisObject
from .analyze_batch_request import AnalyzeBatchRequest
from .batch_analysis_object import BatchAnalysisObject
from .batch_call_data import BatchCallData
from .batch_details_response import BatchDetailsResponse
from .batch_object import BatchObject
from .batch_params import BatchParams
from .call_data import CallData
from .call_length import CallLength
from .call_length_summary import CallLengthSummary
from .call_params import CallParams
from .list_batches_response import ListBatchesResponse
from .queue_status import QueueStatus
from .retrieve_batch_analysis_response import RetrieveBatchAnalysisResponse
from .send_batch_request import SendBatchRequest
from .send_batch_response import SendBatchResponse
from .status_enum import StatusEnum
from .stop_active_batch_response import StopActiveBatchResponse

__all__ = [
    "AnalysisObject",
    "AnalyzeBatchRequest",
    "BatchAnalysisObject",
    "BatchCallData",
    "BatchDetailsResponse",
    "BatchObject",
    "BatchParams",
    "CallData",
    "CallLength",
    "CallLengthSummary",
    "CallParams",
    "ListBatchesResponse",
    "QueueStatus",
    "RetrieveBatchAnalysisResponse",
    "SendBatchRequest",
    "SendBatchResponse",
    "StatusEnum",
    "StopActiveBatchResponse",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime
from .voice_settings_data import VoiceSettingsData

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class UpdateVoicePresetRequest(pydantic.BaseModel):
    voice_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name for the new voice preset.
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    A description of the voice preset.
    """

    voice_id: typing.Optional[int] = pydantic.Field(default=None)
    """
    The existing `voice_id` that will be mapped to the new voice preset.
    """

    reduce_latency: typing.Optional[bool] = pydantic.Field(default=None)
    """
    The current `reduce_latency` setting that will be mapped to the new voice preset.
    """

    interruption_threshold: typing.Optional[int] = pydantic.Field(default=None)
    """
    The threshold for voice interruption settings.
    
    If not provided, the default value is `50`.
    
    Reset to default by setting to `null`.
    """

    language: typing.Optional[str] = pydantic.Field(default=None)
    """
    The language code for the voice preset.
    
    Reset to default by setting to `null`.
    """

    voice_settings: typing.Optional[VoiceSettingsData] = pydantic.Field(default=None)
    """
    A JSON object containing specific settings for the voice.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .types import (
    CreateVoicePresetRequest,
    CreateVoicePresetResponse,
    DeleteVoicePresetResponse,
    GenerateVoiceSampleRequest,
    GenerateVoiceSampleResponse,
    ListVoiceOptionsResponse,
    OptimizeFor,
    Preset,
    PublishVoicePresetResponse,
    RetrieveVoicePresetResponse,
    RetrievedVoice,
    UpdateVoicePresetRequest,
    UpdateVoicePresetResponse,
    Voice,
    VoiceSettingsData,
    VoiceUpdates,
)

__all__ = [
    "CreateVoicePresetRequest",
    "CreateVoicePresetResponse",
    "DeleteVoicePresetResponse",
    "GenerateVoiceSampleRequest",
    "GenerateVoiceSampleResponse",
    "ListVoiceOptionsResponse",
    "OptimizeFor",
    "Preset",
    "PublishVoicePresetResponse",
    "RetrieveVoicePresetResponse",
    "RetrievedVoice",
    "UpdateVoicePresetRequest",
    "UpdateVoicePresetResponse",
    "Voice",
    "VoiceSettingsData",
    "VoiceUpdates",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class UpdateInboundResponse(pydantic.BaseModel):
    status: str = pydantic.Field()
    """
    Whether the update was successful or not - will be `success` or `error`.
    """

    message: str = pydantic.Field()
    """
    A message describing the status of the update.
    """

    updates: typing.Optional[typing.Dict[str, typing.Any]] = pydantic.Field(default=None)
    """
    An object containing the updated settings for the inbound number.
    """

    failed_updates: typing.Optional[typing.Dict[str, str]] = pydantic.Field(default=None)
    """
    If the update was unsuccessful, this will contain the settings that failed to update. Useful to determine how your request is being interpreted on our end.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .types import CallId, CustomTool, ErrorBody, ErrorDetail, PhoneNumber, Status, Tools, VoiceId, Webhook
from .errors import BadRequestError, ServerError, UnauthorizedError

__all__ = [
    "BadRequestError",
    "CallId",
    "CustomTool",
    "ErrorBody",
    "ErrorDetail",
    "PhoneNumber",
    "ServerError",
    "Status",
    "Tools",
    "UnauthorizedError",
    "VoiceId",
    "Webhook",
]



# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

from ..commons.errors.server_error import ServerError
from ..commons.errors.unauthorized_error import UnauthorizedError
from ..commons.types.error_body import ErrorBody
from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.jsonable_encoder import jsonable_encoder
from ..core.remove_none_from_dict import remove_none_from_dict
from ..core.request_options import RequestOptions
from .types.create_voice_preset_request import CreateVoicePresetRequest
from .types.create_voice_preset_response import CreateVoicePresetResponse
from .types.delete_voice_preset_response import DeleteVoicePresetResponse
from .types.generate_voice_sample_request import GenerateVoiceSampleRequest
from .types.generate_voice_sample_response import GenerateVoiceSampleResponse
from .types.list_voice_options_response import ListVoiceOptionsResponse
from .types.publish_voice_preset_response import PublishVoicePresetResponse
from .types.retrieve_voice_preset_response import RetrieveVoicePresetResponse
from .types.update_voice_preset_request import UpdateVoicePresetRequest
from .types.update_voice_preset_response import UpdateVoicePresetResponse

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class VoicesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        *,
        default_only: typing.Optional[bool] = None,
        presets_only: typing.Optional[bool] = None,
        voices_only: typing.Optional[bool] = None,
        is_custom: typing.Optional[bool] = None,
        name: typing.Optional[str] = None,
        voice_name: typing.Optional[str] = None,
        voice_id: typing.Optional[int] = None,
        reduce_latency: typing.Optional[bool] = None,
        public: typing.Optional[bool] = None,
        editable: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListVoiceOptionsResponse:
        """
        Retrieves all available voices for your account, including custom and default presets.

        Parameters:
            - default_only: typing.Optional[bool]. Filter to show only default presets, excluding user-created presets.

            - presets_only: typing.Optional[bool]. Filter to show only presets.

            - voices_only: typing.Optional[bool]. Filter to show only voices.

            - is_custom: typing.Optional[bool]. Filter for custom voices.

            - name: typing.Optional[str]. Filters the voices array by the name of the voice.

            - voice_name: typing.Optional[str]. Filters the presets array by the name of the voice.

            - voice_id: typing.Optional[int]. Filters the presets array by the ID of the voice.

            - reduce_latency: typing.Optional[bool]. Filter by latency reduction option. Use in tandem with `reduce_latency`to simulate exactly which voice your agent would use.

            - public: typing.Optional[bool]. Filter to show only public presets, or set to `false` to exclude them.

            - editable: typing.Optional[bool]. Filter to show only editable presets.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/voices"),
            params=jsonable_encoder(
                remove_none_from_dict(
                    {
                        "default_only": default_only,
                        "presets_only": presets_only,
                        "voices_only": voices_only,
                        "is_custom": is_custom,
                        "name": name,
                        "voice_name": voice_name,
                        "voice_id": voice_id,
                        "reduce_latency": reduce_latency,
                        "public": public,
                        "editable": editable,
                        **(
                            request_options.get("additional_query_parameters", {})
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ListVoiceOptionsResponse, _response.json())  # type: ignore
        if _response.status_code == 500:
            raise ServerError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def retrieve(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> RetrieveVoicePresetResponse:
        """
        Retrieves detailed information about a voice preset for the given id.

        Parameters:
            - id: str. The unique identifier for the voice preset.

                       Used within your call like `voice: "ff2c405b-3dba-41e0-9261-bc8ee3f91f46"` instead of `voice_id: ..., reduce_latency: ...`.
            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"v1/voices/{jsonable_encoder(id)}"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(RetrieveVoicePresetResponse, _response.json())  # type: ignore
        if _response.status_code == 500:
            raise ServerError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self, *, request: CreateVoicePresetRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> CreateVoicePresetResponse:
        """
        Creates a new voice preset by bundling various voice settings into one entity.

        Parameters:
            - request: CreateVoicePresetRequest.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/voices"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(CreateVoicePresetResponse, _response.json())  # type: ignore
        if _response.status_code == 500:
            raise ServerError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self,
        voice_preset_id: str,
        *,
        request: UpdateVoicePresetRequest,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateVoicePresetResponse:
        """
        Updates the settings for a voice preset.

        Parameters:
            - voice_preset_id: str. The `id` of the voice preset to update.

                                    Note: Public voices cannot be altered once published.
            - request: UpdateVoicePresetRequest.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"v1/voices/${jsonable_encoder(voice_preset_id)}"
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(UpdateVoicePresetResponse, _response.json())  # type: ignore
        if _response.status_code == 500:
            raise ServerError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(
        self, voice_preset_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteVoicePresetResponse:
        """
        Deletes a voice preset.

        Parameters:
            - voice_preset_id: str. The unique identifier for the voice preset to be deleted.

                                    Note:

                                    - This voice preset must have been created with your account
                                    - Default or other user's voice presets cannot be deleted.
                                    - Public voice presets can be deleted, but not modified.
                                    - This will not delete cloned voice presets, only the original.
            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"v1/voices/{jsonable_encoder(voice_preset_id)}/delete"
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))
            if request_options is not None
            else None,
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(DeleteVoicePresetResponse, _response.json())  # type: ignore
        if _response.status_code == 500:
            raise ServerError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def publish(
        self, voice_preset_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> PublishVoicePresetResponse:
        """
        Publishes your voice preset, making it publicly available.

        Parameters:
            - voice_preset_id: str. The unique identifier for the voice preset to be published.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"v1/voices/{jsonable_encoder(voice_preset_id)}/publish"
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))
            if request_options is not None
            else None,
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(PublishVoicePresetResponse, _response.json())  # type: ignore
        if _response.status_code == 500:
            raise ServerError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def generate(
        self,
        voice_preset_id: str,
        *,
        request: GenerateVoiceSampleRequest,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GenerateVoiceSampleResponse:
        """
        Generates an audio sample from a specified voice preset, with options to override default settings for tuning and testing.

        Parameters:
            - voice_preset_id: str. The unique identifier for the voice preset to be used for generating the sample.

            - request: GenerateVoiceSampleRequest.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"v1/voices/{jsonable_encoder(voice_preset_id)}/sample"
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(GenerateVoiceSampleResponse, _response.json())  # type: ignore
        if _response.status_code == 500:
            raise ServerError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncVoicesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        *,
        default_only: typing.Optional[bool] = None,
        presets_only: typing.Optional[bool] = None,
        voices_only: typing.Optional[bool] = None,
        is_custom: typing.Optional[bool] = None,
        name: typing.Optional[str] = None,
        voice_name: typing.Optional[str] = None,
        voice_id: typing.Optional[int] = None,
        reduce_latency: typing.Optional[bool] = None,
        public: typing.Optional[bool] = None,
        editable: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListVoiceOptionsResponse:
        """
        Retrieves all available voices for your account, including custom and default presets.

        Parameters:
            - default_only: typing.Optional[bool]. Filter to show only default presets, excluding user-created presets.

            - presets_only: typing.Optional[bool]. Filter to show only presets.

            - voices_only: typing.Optional[bool]. Filter to show only voices.

            - is_custom: typing.Optional[bool]. Filter for custom voices.

            - name: typing.Optional[str]. Filters the voices array by the name of the voice.

            - voice_name: typing.Optional[str]. Filters the presets array by the name of the voice.

            - voice_id: typing.Optional[int]. Filters the presets array by the ID of the voice.

            - reduce_latency: typing.Optional[bool]. Filter by latency reduction option. Use in tandem with `reduce_latency`to simulate exactly which voice your agent would use.

            - public: typing.Optional[bool]. Filter to show only public presets, or set to `false` to exclude them.

            - editable: typing.Optional[bool]. Filter to show only editable presets.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/voices"),
            params=jsonable_encoder(
                remove_none_from_dict(
                    {
                        "default_only": default_only,
                        "presets_only": presets_only,
                        "voices_only": voices_only,
                        "is_custom": is_custom,
                        "name": name,
                        "voice_name": voice_name,
                        "voice_id": voice_id,
                        "reduce_latency": reduce_latency,
                        "public": public,
                        "editable": editable,
                        **(
                            request_options.get("additional_query_parameters", {})
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ListVoiceOptionsResponse, _response.json())  # type: ignore
        if _response.status_code == 500:
            raise ServerError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def retrieve(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> RetrieveVoicePresetResponse:
        """
        Retrieves detailed information about a voice preset for the given id.

        Parameters:
            - id: str. The unique identifier for the voice preset.

                       Used within your call like `voice: "ff2c405b-3dba-41e0-9261-bc8ee3f91f46"` instead of `voice_id: ..., reduce_latency: ...`.
            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"v1/voices/{jsonable_encoder(id)}"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(RetrieveVoicePresetResponse, _response.json())  # type: ignore
        if _response.status_code == 500:
            raise ServerError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self, *, request: CreateVoicePresetRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> CreateVoicePresetResponse:
        """
        Creates a new voice preset by bundling various voice settings into one entity.

        Parameters:
            - request: CreateVoicePresetRequest.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/voices"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(CreateVoicePresetResponse, _response.json())  # type: ignore
        if _response.status_code == 500:
            raise ServerError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self,
        voice_preset_id: str,
        *,
        request: UpdateVoicePresetRequest,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateVoicePresetResponse:
        """
        Updates the settings for a voice preset.

        Parameters:
            - voice_preset_id: str. The `id` of the voice preset to update.

                                    Note: Public voices cannot be altered once published.
            - request: UpdateVoicePresetRequest.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"v1/voices/${jsonable_encoder(voice_preset_id)}"
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(UpdateVoicePresetResponse, _response.json())  # type: ignore
        if _response.status_code == 500:
            raise ServerError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(
        self, voice_preset_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteVoicePresetResponse:
        """
        Deletes a voice preset.

        Parameters:
            - voice_preset_id: str. The unique identifier for the voice preset to be deleted.

                                    Note:

                                    - This voice preset must have been created with your account
                                    - Default or other user's voice presets cannot be deleted.
                                    - Public voice presets can be deleted, but not modified.
                                    - This will not delete cloned voice presets, only the original.
            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"v1/voices/{jsonable_encoder(voice_preset_id)}/delete"
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))
            if request_options is not None
            else None,
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(DeleteVoicePresetResponse, _response.json())  # type: ignore
        if _response.status_code == 500:
            raise ServerError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def publish(
        self, voice_preset_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> PublishVoicePresetResponse:
        """
        Publishes your voice preset, making it publicly available.

        Parameters:
            - voice_preset_id: str. The unique identifier for the voice preset to be published.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"v1/voices/{jsonable_encoder(voice_preset_id)}/publish"
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))
            if request_options is not None
            else None,
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(PublishVoicePresetResponse, _response.json())  # type: ignore
        if _response.status_code == 500:
            raise ServerError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def generate(
        self,
        voice_preset_id: str,
        *,
        request: GenerateVoiceSampleRequest,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GenerateVoiceSampleResponse:
        """
        Generates an audio sample from a specified voice preset, with options to override default settings for tuning and testing.

        Parameters:
            - voice_preset_id: str. The unique identifier for the voice preset to be used for generating the sample.

            - request: GenerateVoiceSampleRequest.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"v1/voices/{jsonable_encoder(voice_preset_id)}/sample"
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(GenerateVoiceSampleResponse, _response.json())  # type: ignore
        if _response.status_code == 500:
            raise ServerError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime
from .call_params import CallParams

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class BatchParams(pydantic.BaseModel):
    id: str = pydantic.Field()
    """
    The unique identifier of the batch - used as the `batch_id` parameter in other API calls.
    """

    campaign_id: typing.Optional[str] = None
    created_at: dt.datetime = pydantic.Field()
    """
    The creation timestamp of the batch.
    """

    label: str = pydantic.Field()
    """
    The label or description of the batch.
    """

    base_prompt: str = pydantic.Field()
    """
    The base prompt used for calls in this batch.
    """

    endpoint_code: str = pydantic.Field()
    """
    The endpoint code used for API integration.
    """

    call_params: CallParams = pydantic.Field()
    """
    An object containing parameters for the calls in the batch.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .call_id import CallId
from .custom_tool import CustomTool
from .error_body import ErrorBody
from .error_detail import ErrorDetail
from .phone_number import PhoneNumber
from .status import Status
from .tools import Tools
from .voice_id import VoiceId
from .webhook import Webhook

__all__ = ["CallId", "CustomTool", "ErrorBody", "ErrorDetail", "PhoneNumber", "Status", "Tools", "VoiceId", "Webhook"]



# This file was auto-generated by Fern from our API Definition.

from .bad_request_error import BadRequestError
from .server_error import ServerError
from .unauthorized_error import UnauthorizedError

__all__ = ["BadRequestError", "ServerError", "UnauthorizedError"]



# This file was auto-generated by Fern from our API Definition.

from .types import (
    DynamicData,
    InterruptionThreshold,
    MethodEnum,
    ModelEnum,
    PronunciationObject,
    ProunciationGuide,
    ResponseData,
    SendCall,
    SendCallResponse,
    Temperature,
    TransferList,
    VoicemailAction,
)
from . import batches, calls, commons, inbound_numbers, outbound_numbers, sms, voices
from .batches import (
    AnalysisObject,
    AnalyzeBatchRequest,
    BatchAnalysisObject,
    BatchCallData,
    BatchDetailsResponse,
    BatchObject,
    BatchParams,
    CallData,
    CallLength,
    CallLengthSummary,
    CallParams,
    ListBatchesResponse,
    QueueStatus,
    RetrieveBatchAnalysisResponse,
    SendBatchRequest,
    SendBatchResponse,
    StatusEnum,
    StopActiveBatchResponse,
)
from .calls import (
    AnalyzeCall,
    AnalyzeCallResponse,
    AnsweredByEnum,
    AudioRecordingResponse,
    CallDetailsResponse,
    CallIdParam,
    Goal,
    ListCallsResponse,
    ListedCallData,
    StopCallResponse,
    Transcript,
    UserEnum,
)
from .commons import (
    BadRequestError,
    CallId,
    CustomTool,
    ErrorBody,
    ErrorDetail,
    PhoneNumber,
    ServerError,
    Status,
    Tools,
    UnauthorizedError,
    VoiceId,
    Webhook,
)
from .environment import BlandAIEnvironment
from .inbound_numbers import (
    CountryCode,
    InboundNumber,
    InboundNumberDetailsResponse,
    InboundNumberRequest,
    InboundNumberResponse,
    ListInboundResponse,
    Model,
    UpdateInbound,
    UpdateInboundResponse,
    VoiceSettings,
)
from .outbound_numbers import ListOutboundResponse, OutboundNumber, OutboundPurchase, OutboundPurchaseResponse
from .sms import (
    A2PStatusEnum,
    CheckStatusRequest,
    CheckStatusResponse,
    GetMessagesRequest,
    GetMessagesResponse,
    LegalStructureEnum,
    RegisterRequest,
    RegisterResponse,
    ToggleReplyEnum,
    ToggleReplyRequest,
    ToggleReplyResponse,
    TrustedUser,
    UpdatePromptRequest,
    UpdatePromptResponse,
    UpdateWebhookRequest,
    UpdateWebhookResponse,
    VerticalEnum,
)
from .voices import (
    CreateVoicePresetRequest,
    CreateVoicePresetResponse,
    DeleteVoicePresetResponse,
    GenerateVoiceSampleRequest,
    GenerateVoiceSampleResponse,
    ListVoiceOptionsResponse,
    OptimizeFor,
    Preset,
    PublishVoicePresetResponse,
    RetrieveVoicePresetResponse,
    RetrievedVoice,
    UpdateVoicePresetRequest,
    UpdateVoicePresetResponse,
    Voice,
    VoiceSettingsData,
    VoiceUpdates,
)

__all__ = [
    "A2PStatusEnum",
    "AnalysisObject",
    "AnalyzeBatchRequest",
    "AnalyzeCall",
    "AnalyzeCallResponse",
    "AnsweredByEnum",
    "AudioRecordingResponse",
    "BadRequestError",
    "BatchAnalysisObject",
    "BatchCallData",
    "BatchDetailsResponse",
    "BatchObject",
    "BatchParams",
    "BlandAIEnvironment",
    "CallData",
    "CallDetailsResponse",
    "CallId",
    "CallIdParam",
    "CallLength",
    "CallLengthSummary",
    "CallParams",
    "CheckStatusRequest",
    "CheckStatusResponse",
    "CountryCode",
    "CreateVoicePresetRequest",
    "CreateVoicePresetResponse",
    "CustomTool",
    "DeleteVoicePresetResponse",
    "DynamicData",
    "ErrorBody",
    "ErrorDetail",
    "GenerateVoiceSampleRequest",
    "GenerateVoiceSampleResponse",
    "GetMessagesRequest",
    "GetMessagesResponse",
    "Goal",
    "InboundNumber",
    "InboundNumberDetailsResponse",
    "InboundNumberRequest",
    "InboundNumberResponse",
    "InterruptionThreshold",
    "LegalStructureEnum",
    "ListBatchesResponse",
    "ListCallsResponse",
    "ListInboundResponse",
    "ListOutboundResponse",
    "ListVoiceOptionsResponse",
    "ListedCallData",
    "MethodEnum",
    "Model",
    "ModelEnum",
    "OptimizeFor",
    "OutboundNumber",
    "OutboundPurchase",
    "OutboundPurchaseResponse",
    "PhoneNumber",
    "Preset",
    "PronunciationObject",
    "ProunciationGuide",
    "PublishVoicePresetResponse",
    "QueueStatus",
    "RegisterRequest",
    "RegisterResponse",
    "ResponseData",
    "RetrieveBatchAnalysisResponse",
    "RetrieveVoicePresetResponse",
    "RetrievedVoice",
    "SendBatchRequest",
    "SendBatchResponse",
    "SendCall",
    "SendCallResponse",
    "ServerError",
    "Status",
    "StatusEnum",
    "StopActiveBatchResponse",
    "StopCallResponse",
    "Temperature",
    "ToggleReplyEnum",
    "ToggleReplyRequest",
    "ToggleReplyResponse",
    "Tools",
    "Transcript",
    "TransferList",
    "TrustedUser",
    "UnauthorizedError",
    "UpdateInbound",
    "UpdateInboundResponse",
    "UpdatePromptRequest",
    "UpdatePromptResponse",
    "UpdateVoicePresetRequest",
    "UpdateVoicePresetResponse",
    "UpdateWebhookRequest",
    "UpdateWebhookResponse",
    "UserEnum",
    "VerticalEnum",
    "Voice",
    "VoiceId",
    "VoiceSettings",
    "VoiceSettingsData",
    "VoiceUpdates",
    "VoicemailAction",
    "Webhook",
    "batches",
    "calls",
    "commons",
    "inbound_numbers",
    "outbound_numbers",
    "sms",
    "voices",
]



# This file was auto-generated by Fern from our API Definition.

from .country_code import CountryCode
from .inbound_number import InboundNumber
from .inbound_number_details_response import InboundNumberDetailsResponse
from .inbound_number_request import InboundNumberRequest
from .inbound_number_response import InboundNumberResponse
from .list_inbound_response import ListInboundResponse
from .model import Model
from .update_inbound import UpdateInbound
from .update_inbound_response import UpdateInboundResponse
from .voice_settings import VoiceSettings

__all__ = [
    "CountryCode",
    "InboundNumber",
    "InboundNumberDetailsResponse",
    "InboundNumberRequest",
    "InboundNumberResponse",
    "ListInboundResponse",
    "Model",
    "UpdateInbound",
    "UpdateInboundResponse",
    "VoiceSettings",
]



# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

from ..commons.errors.server_error import ServerError
from ..commons.errors.unauthorized_error import UnauthorizedError
from ..commons.types.error_body import ErrorBody
from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.jsonable_encoder import jsonable_encoder
from ..core.remove_none_from_dict import remove_none_from_dict
from ..core.request_options import RequestOptions
from .types.check_status_request import CheckStatusRequest
from .types.check_status_response import CheckStatusResponse
from .types.get_messages_request import GetMessagesRequest
from .types.get_messages_response import GetMessagesResponse
from .types.register_request import RegisterRequest
from .types.register_response import RegisterResponse
from .types.toggle_reply_request import ToggleReplyRequest
from .types.toggle_reply_response import ToggleReplyResponse
from .types.update_prompt_request import UpdatePromptRequest
from .types.update_prompt_response import UpdatePromptResponse
from .types.update_webhook_request import UpdateWebhookRequest
from .types.update_webhook_response import UpdateWebhookResponse

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class SmsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def register(
        self, *, request: RegisterRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> RegisterResponse:
        """
        This documentation provides detailed information on how to register an Application-to-Person (A2P) brand by making a POST request to our API. The process involves submitting your brand's details for registration and verification purposes.

        A2P Registration is required _for_ all businesses who wish to send SMS. There can be signifcant fines for any non compliant messages. A2P Registration can take 2 days -> 2 Weeks.

        Parameters:
            - request: RegisterRequest.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/sms/register"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(RegisterResponse, _response.json())  # type: ignore
        if _response.status_code == 500:
            raise ServerError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def check(
        self, *, request: CheckStatusRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> CheckStatusResponse:
        """
        Check the status of an A2P registration.

        Parameters:
            - request: CheckStatusRequest.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/sms/register/status"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(CheckStatusResponse, _response.json())  # type: ignore
        if _response.status_code == 500:
            raise ServerError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(
        self, *, request: GetMessagesRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> GetMessagesResponse:
        """
        Get the list of SMS messages for a given conversation.

        Parameters:
            - request: GetMessagesRequest.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/sms/messages/get"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(GetMessagesResponse, _response.json())  # type: ignore
        if _response.status_code == 500:
            raise ServerError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def toggle(
        self, *, request: ToggleReplyRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> ToggleReplyResponse:
        """
        Turn on or off the AI replying for a given phone number.

        Parameters:
            - request: ToggleReplyRequest.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/sms/toggle"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ToggleReplyResponse, _response.json())  # type: ignore
        if _response.status_code == 500:
            raise ServerError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update_prompt(
        self, *, request: UpdatePromptRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> UpdatePromptResponse:
        """
        Purchase and configure a new inbound phone number. ($15/mo. subscription using your stored payment method).

        Parameters:
            - request: UpdatePromptRequest.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/sms/prompt/update"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(UpdatePromptResponse, _response.json())  # type: ignore
        if _response.status_code == 500:
            raise ServerError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update_webhook(
        self, *, request: UpdateWebhookRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> UpdateWebhookResponse:
        """
        Update the webhook for a given phone number.

        Parameters:
            - request: UpdateWebhookRequest.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/sms/webhook/update"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(UpdateWebhookResponse, _response.json())  # type: ignore
        if _response.status_code == 500:
            raise ServerError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncSmsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def register(
        self, *, request: RegisterRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> RegisterResponse:
        """
        This documentation provides detailed information on how to register an Application-to-Person (A2P) brand by making a POST request to our API. The process involves submitting your brand's details for registration and verification purposes.

        A2P Registration is required _for_ all businesses who wish to send SMS. There can be signifcant fines for any non compliant messages. A2P Registration can take 2 days -> 2 Weeks.

        Parameters:
            - request: RegisterRequest.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/sms/register"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(RegisterResponse, _response.json())  # type: ignore
        if _response.status_code == 500:
            raise ServerError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def check(
        self, *, request: CheckStatusRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> CheckStatusResponse:
        """
        Check the status of an A2P registration.

        Parameters:
            - request: CheckStatusRequest.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/sms/register/status"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(CheckStatusResponse, _response.json())  # type: ignore
        if _response.status_code == 500:
            raise ServerError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self, *, request: GetMessagesRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> GetMessagesResponse:
        """
        Get the list of SMS messages for a given conversation.

        Parameters:
            - request: GetMessagesRequest.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/sms/messages/get"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(GetMessagesResponse, _response.json())  # type: ignore
        if _response.status_code == 500:
            raise ServerError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def toggle(
        self, *, request: ToggleReplyRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> ToggleReplyResponse:
        """
        Turn on or off the AI replying for a given phone number.

        Parameters:
            - request: ToggleReplyRequest.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/sms/toggle"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ToggleReplyResponse, _response.json())  # type: ignore
        if _response.status_code == 500:
            raise ServerError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update_prompt(
        self, *, request: UpdatePromptRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> UpdatePromptResponse:
        """
        Purchase and configure a new inbound phone number. ($15/mo. subscription using your stored payment method).

        Parameters:
            - request: UpdatePromptRequest.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/sms/prompt/update"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(UpdatePromptResponse, _response.json())  # type: ignore
        if _response.status_code == 500:
            raise ServerError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update_webhook(
        self, *, request: UpdateWebhookRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> UpdateWebhookResponse:
        """
        Update the webhook for a given phone number.

        Parameters:
            - request: UpdateWebhookRequest.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/sms/webhook/update"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(UpdateWebhookResponse, _response.json())  # type: ignore
        if _response.status_code == 500:
            raise ServerError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import os
import typing
import urllib.parse
from json.decoder import JSONDecodeError

import httpx

from .batches.client import AsyncBatchesClient, BatchesClient
from .calls.client import AsyncCallsClient, CallsClient
from .commons.errors.server_error import ServerError
from .commons.errors.unauthorized_error import UnauthorizedError
from .commons.types.error_body import ErrorBody
from .core.api_error import ApiError
from .core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .core.jsonable_encoder import jsonable_encoder
from .core.remove_none_from_dict import remove_none_from_dict
from .core.request_options import RequestOptions
from .environment import BlandAIEnvironment
from .inbound_numbers.client import AsyncInboundNumbersClient, InboundNumbersClient
from .outbound_numbers.client import AsyncOutboundNumbersClient, OutboundNumbersClient
from .sms.client import AsyncSmsClient, SmsClient
from .types.send_call import SendCall
from .types.send_call_response import SendCallResponse
from .voices.client import AsyncVoicesClient, VoicesClient

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class BlandAI:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propogate to these functions.

    Parameters:
        - base_url: typing.Optional[str]. The base url to use for requests from the client.

        - environment: BlandAIEnvironment. The environment to use for requests from the client. from .environment import BlandAIEnvironment

                                           Defaults to BlandAIEnvironment.PRODUCTION

        - api_key: typing.Optional[str].

        - timeout: typing.Optional[float]. The timeout to be used, in seconds, for requests by default the timeout is 60 seconds.

        - httpx_client: typing.Optional[httpx.Client]. The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.
    ---
    from blandai.client import BlandAI

    client = BlandAI(
        api_key="YOUR_API_KEY",
    )
    """

    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: BlandAIEnvironment = BlandAIEnvironment.PRODUCTION,
        api_key: typing.Optional[str] = os.getenv("BLAND_API_KEY"),
        timeout: typing.Optional[float] = 60,
        httpx_client: typing.Optional[httpx.Client] = None,
    ):
        if api_key is None:
            raise ApiError(body="The client must be instantiated be either passing in api_key or setting BLAND_API_KEY")
        self._client_wrapper = SyncClientWrapper(
            base_url=_get_base_url(base_url=base_url, environment=environment),
            api_key=api_key,
            httpx_client=httpx.Client(timeout=timeout) if httpx_client is None else httpx_client,
        )
        self.batches = BatchesClient(client_wrapper=self._client_wrapper)
        self.calls = CallsClient(client_wrapper=self._client_wrapper)
        self.inbound_numbers = InboundNumbersClient(client_wrapper=self._client_wrapper)
        self.outbound_numbers = OutboundNumbersClient(client_wrapper=self._client_wrapper)
        self.sms = SmsClient(client_wrapper=self._client_wrapper)
        self.voices = VoicesClient(client_wrapper=self._client_wrapper)

    def call(self, *, request: SendCall, request_options: typing.Optional[RequestOptions] = None) -> SendCallResponse:
        """
        Send an AI phone call with a custom objective and actions.

        Parameters:
            - request: SendCall.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from blandai import PronunciationObject, SendCall
        from blandai.client import BlandAI

        client = BlandAI(
            api_key="YOUR_API_KEY",
        )
        client.call(
            request=SendCall(
                phone_number="29382721828",
                task="Would love for you to check out our AI API!",
                temperature=0.5,
                transfer_list={
                    "default": "+1234567890",
                    "sales": "+1234567890",
                    "support": "+1234567890",
                    "billing": "+1234567890",
                },
                model="enhanced",
                pronunciation_guide=[
                    PronunciationObject(
                        word="API",
                        pronunciation="A P I",
                    ),
                    PronunciationObject(
                        word="AI",
                        pronunciation="A I",
                    ),
                ],
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/calls"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(SendCallResponse, _response.json())  # type: ignore
        if _response.status_code == 500:
            raise ServerError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncBlandAI:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propogate to these functions.

    Parameters:
        - base_url: typing.Optional[str]. The base url to use for requests from the client.

        - environment: BlandAIEnvironment. The environment to use for requests from the client. from .environment import BlandAIEnvironment

                                           Defaults to BlandAIEnvironment.PRODUCTION

        - api_key: typing.Optional[str].

        - timeout: typing.Optional[float]. The timeout to be used, in seconds, for requests by default the timeout is 60 seconds.

        - httpx_client: typing.Optional[httpx.AsyncClient]. The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.
    ---
    from blandai.client import AsyncBlandAI

    client = AsyncBlandAI(
        api_key="YOUR_API_KEY",
    )
    """

    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: BlandAIEnvironment = BlandAIEnvironment.PRODUCTION,
        api_key: typing.Optional[str] = os.getenv("BLAND_API_KEY"),
        timeout: typing.Optional[float] = 60,
        httpx_client: typing.Optional[httpx.AsyncClient] = None,
    ):
        if api_key is None:
            raise ApiError(body="The client must be instantiated be either passing in api_key or setting BLAND_API_KEY")
        self._client_wrapper = AsyncClientWrapper(
            base_url=_get_base_url(base_url=base_url, environment=environment),
            api_key=api_key,
            httpx_client=httpx.AsyncClient(timeout=timeout) if httpx_client is None else httpx_client,
        )
        self.batches = AsyncBatchesClient(client_wrapper=self._client_wrapper)
        self.calls = AsyncCallsClient(client_wrapper=self._client_wrapper)
        self.inbound_numbers = AsyncInboundNumbersClient(client_wrapper=self._client_wrapper)
        self.outbound_numbers = AsyncOutboundNumbersClient(client_wrapper=self._client_wrapper)
        self.sms = AsyncSmsClient(client_wrapper=self._client_wrapper)
        self.voices = AsyncVoicesClient(client_wrapper=self._client_wrapper)

    async def call(
        self, *, request: SendCall, request_options: typing.Optional[RequestOptions] = None
    ) -> SendCallResponse:
        """
        Send an AI phone call with a custom objective and actions.

        Parameters:
            - request: SendCall.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from blandai import PronunciationObject, SendCall
        from blandai.client import AsyncBlandAI

        client = AsyncBlandAI(
            api_key="YOUR_API_KEY",
        )
        await client.call(
            request=SendCall(
                phone_number="29382721828",
                task="Would love for you to check out our AI API!",
                temperature=0.5,
                transfer_list={
                    "default": "+1234567890",
                    "sales": "+1234567890",
                    "support": "+1234567890",
                    "billing": "+1234567890",
                },
                model="enhanced",
                pronunciation_guide=[
                    PronunciationObject(
                        word="API",
                        pronunciation="A P I",
                    ),
                    PronunciationObject(
                        word="AI",
                        pronunciation="A I",
                    ),
                ],
            ),
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/calls"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(SendCallResponse, _response.json())  # type: ignore
        if _response.status_code == 500:
            raise ServerError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


def _get_base_url(*, base_url: typing.Optional[str] = None, environment: BlandAIEnvironment) -> str:
    if base_url is not None:
        return base_url
    elif environment is not None:
        return environment.value
    else:
        raise Exception("Please pass in either base_url or environment to construct the client")



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime
from .optimize_for import OptimizeFor

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class VoiceSettingsData(pydantic.BaseModel):
    speed: typing.Optional[int] = pydantic.Field(default=None)
    """
    Multiplies the speed of the voice (only `reduce_latency: true` voices).
    """

    interruption_threshold: typing.Optional[int] = pydantic.Field(default=None)
    """
    Delay before responding (in milliseconds).
    """

    stability: typing.Optional[int] = pydantic.Field(default=None)
    """
    Lower for more emotion, higher for more stability.
    """

    similarity: typing.Optional[int] = pydantic.Field(default=None)
    """
    Match cloned voice more closely.
    """

    speech_flexibility: typing.Optional[int] = pydantic.Field(default=None)
    """
    Flexible pronunciation.
    """

    optimize_for: typing.Optional[OptimizeFor] = pydantic.Field(default=None)
    """
    The optimization setting for the voice.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .types import (
    CountryCode,
    InboundNumber,
    InboundNumberDetailsResponse,
    InboundNumberRequest,
    InboundNumberResponse,
    ListInboundResponse,
    Model,
    UpdateInbound,
    UpdateInboundResponse,
    VoiceSettings,
)

__all__ = [
    "CountryCode",
    "InboundNumber",
    "InboundNumberDetailsResponse",
    "InboundNumberRequest",
    "InboundNumberResponse",
    "ListInboundResponse",
    "Model",
    "UpdateInbound",
    "UpdateInboundResponse",
    "VoiceSettings",
]



# This file was auto-generated by Fern from our API Definition.

import typing

LegalStructureEnum = typing.Literal[
    "Partnership", "Limited Liability Corporation", "Co-operative", "Non-profit Corporation", "Corporation"
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime
from .analysis_object import AnalysisObject
from .batch_params import BatchParams
from .call_data import CallData

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class BatchDetailsResponse(pydantic.BaseModel):
    batch_params: BatchParams = pydantic.Field()
    """
    An object containing parameters and settings for the batch.
    """

    analysis: AnalysisObject = pydantic.Field()
    """
    An object containing analysis data for the batch.
    """

    call_data: typing.List[CallData] = pydantic.Field()
    """
    An array of objects, each representing individual call data.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .types import ListOutboundResponse, OutboundNumber, OutboundPurchase, OutboundPurchaseResponse

__all__ = ["ListOutboundResponse", "OutboundNumber", "OutboundPurchase", "OutboundPurchaseResponse"]
